@@ -2,10 +2,8 @@ package controller
 
 import (
 	"fmt"
-	"net"
 	"strings"
 
-	"github.com/coreos/go-iptables/iptables"
 	"github.com/urfave/cli"
 	"k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -15,7 +13,6 @@ import (
 	"github.com/ovn-org/ovn-kubernetes/go-controller/hybrid-overlay/pkg/types"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/factory"
-	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/ovn"
 	ovntest "github.com/ovn-org/ovn-kubernetes/go-controller/pkg/testing"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 
@@ -37,10 +34,6 @@ func addNodeSetupCmds(fexec *ovntest.FakeExec, nodeName string) (string, string)
 		Output: testNodeSubnet,
 	})
 	addGetPortAddressesCmds(fexec, nodeName, testDRMAC, testNodeIP)
-	fexec.AddFakeCmd(&ovntest.ExpectedCmd{
-		Cmd: "ovs-vsctl --timeout=15 br-exists br-ext",
-		Err: fmt.Errorf("bridge br-ext not found"),
-	})
 	fexec.AddFakeCmdsNoOutputNoError([]string{
 		"ovs-vsctl --timeout=15 --may-exist add-br br-ext -- set Bridge br-ext fail_mode=secure",
 	})
@@ -51,16 +44,16 @@ func addNodeSetupCmds(fexec *ovntest.FakeExec, nodeName string) (string, string)
 	fexec.AddFakeCmdsNoOutputNoError([]string{
 		"ovs-vsctl --timeout=15 set bridge br-ext other-config:hwaddr=" + testOVSMAC,
 		"ip link set br-ext up",
-		"ovs-vsctl --timeout=15 --may-exist add-port br-int int -- --may-exist add-port br-ext ext -- set Interface int type=patch options:peer=ext external-ids:iface-id=int-" + nodeName + " -- set Interface ext type=patch options:peer=int ofport_request=11",
-		"ovs-ofctl -O openflow13 add-flow br-ext table=0, priority=0, actions=drop",
+		"ovs-vsctl --timeout=15 --may-exist add-port br-int int -- --may-exist add-port br-ext ext -- set Interface int type=patch options:peer=ext external-ids:iface-id=int-" + nodeName + " -- set Interface ext type=patch options:peer=int",
+		"ovs-ofctl add-flow br-ext table=0,priority=0,actions=drop",
 	})
 	testDRMACRaw := strings.Replace(testDRMAC, ":", "", -1)
-	testNodeIPRaw := getIPAsHexString(net.ParseIP(testNodeIP))
+	testNodeIPRaw := getIPAsHexString(ovntest.MustParseIP(testNodeIP))
 	fexec.AddFakeCmdsNoOutputNoError([]string{
-		"ovs-ofctl -O openflow13 add-flow br-ext table=0, priority=100, in_port=11, arp, arp_tpa=" + testNodeIP + ", actions=move:NXM_OF_ETH_SRC[]->NXM_OF_ETH_DST[],mod_dl_src:" + testDRMAC + ",load:0x2->NXM_OF_ARP_OP[],move:NXM_NX_ARP_SHA[]->NXM_NX_ARP_THA[],move:NXM_OF_ARP_SPA[]->NXM_OF_ARP_TPA[],load:0x" + testDRMACRaw + "->NXM_NX_ARP_SHA[],load:0x" + testNodeIPRaw + "->NXM_OF_ARP_SPA[],IN_PORT",
-		`ovs-vsctl --timeout=15 --may-exist add-port br-ext ext-vxlan -- set interface ext-vxlan ofport_request=1 type=vxlan options:remote_ip="flow" options:key="flow"`,
-		"ovs-ofctl -O openflow13 add-flow br-ext table=0, priority=100, in_port=ext-vxlan, ip, nw_dst=" + testNodeSubnet + ", dl_dst=" + testDRMAC + ", actions=goto_table:10",
-		"ovs-ofctl -O openflow13 add-flow br-ext table=10, priority=0, actions=drop",
+		"ovs-ofctl add-flow br-ext table=0,priority=100,in_port=ext,arp,arp_tpa=" + testNodeIP + ",actions=move:NXM_OF_ETH_SRC[]->NXM_OF_ETH_DST[],mod_dl_src:" + testDRMAC + ",load:0x2->NXM_OF_ARP_OP[],move:NXM_NX_ARP_SHA[]->NXM_NX_ARP_THA[],move:NXM_OF_ARP_SPA[]->NXM_OF_ARP_TPA[],load:0x" + testDRMACRaw + "->NXM_NX_ARP_SHA[],load:0x" + testNodeIPRaw + "->NXM_OF_ARP_SPA[],IN_PORT",
+		`ovs-vsctl --timeout=15 --may-exist add-port br-ext ext-vxlan -- set interface ext-vxlan type=vxlan options:remote_ip="flow" options:key="flow"`,
+		"ovs-ofctl add-flow br-ext table=0,priority=100,in_port=ext-vxlan,ip,nw_dst=" + testNodeSubnet + ",dl_dst=" + testDRMAC + ",actions=goto_table:10",
+		"ovs-ofctl add-flow br-ext table=10,priority=0,actions=drop",
 	})
 	return testNodeIP, testDRMAC
 }
@@ -85,7 +78,7 @@ func createNode(name, os, ip string, annotations map[string]string) *v1.Node {
 func createPod(namespace, name, node, podIP, podMAC string) *v1.Pod {
 	annotations := map[string]string{}
 	if podIP != "" || podMAC != "" {
-		_, ipn, _ := net.ParseCIDR(podIP)
+		ipn := ovntest.MustParseIPNet(podIP)
 		gatewayIP := util.NextIP(ipn.IP)
 		annotations[util.OvnPodAnnotationName] = fmt.Sprintf(`{"default": {"ip_address":"` + podIP + `", "mac_address":"` + podMAC + `", "gateway_ip": "` + gatewayIP.String() + `"}}`)
 	}
@@ -108,30 +101,24 @@ func createPod(namespace, name, node, podIP, podMAC string) *v1.Pod {
 var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 	var (
 		app   *cli.App
-		ipt   *util.FakeIPTables
 		fexec *ovntest.FakeExec
 	)
 	const thisNode string = "mynode"
 
 	BeforeEach(func() {
 		// Restore global default values before each testcase
-		config.RestoreDefaultConfig()
+		config.PrepareTestConfig()
 
 		app = cli.NewApp()
 		app.Name = "test"
 		app.Flags = config.Flags
 
-		var err error
-		ipt, err = util.NewFakeWithProtocol(iptables.ProtocolIPv4)
-		Expect(err).NotTo(HaveOccurred())
-		util.SetIPTablesHelper(iptables.ProtocolIPv4, ipt)
-
 		fexec = ovntest.NewFakeExec()
-		err = util.SetExec(fexec)
+		err := util.SetExec(fexec)
 		Expect(err).NotTo(HaveOccurred())
 	})
 
-	It("does not set up tunnels for non-Windows nodes without annotations", func() {
+	It("does not set up tunnels for non-hybrid-overlay nodes without annotations", func() {
 		app.Action = func(ctx *cli.Context) error {
 			const (
 				node1Name string = "node1"
@@ -168,11 +155,14 @@ var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 			return nil
 		}
 
-		err := app.Run([]string{app.Name})
+		err := app.Run([]string{
+			app.Name,
+			hoNodeCliArg,
+		})
 		Expect(err).NotTo(HaveOccurred())
 	})
 
-	It("does not set up tunnels for non-Windows nodes with subnet annotations", func() {
+	It("does not set up tunnels for non-hybrid-overlay nodes with subnet annotations", func() {
 		app.Action = func(ctx *cli.Context) error {
 			const (
 				node1Name   string = "node1"
@@ -180,12 +170,16 @@ var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 				node1IP     string = "10.0.0.2"
 			)
 
+			subnetAnnotations, err := util.CreateNodeHostSubnetAnnotation(ovntest.MustParseIPNet(node1Subnet))
+			Expect(err).NotTo(HaveOccurred())
+			annotations := make(map[string]string)
+			for k, v := range subnetAnnotations {
+				annotations[k] = fmt.Sprintf("%s", v)
+			}
+
 			fakeClient := fake.NewSimpleClientset(&v1.NodeList{
 				Items: []v1.Node{
-					*createNode(node1Name, "linux", node1IP, map[string]string{
-						types.HybridOverlayNodeSubnet: node1Subnet,
-						ovn.OvnNodeSubnets:            fmt.Sprintf(`{"default": "%s"}`, node1Subnet),
-					}),
+					*createNode(node1Name, "linux", node1IP, annotations),
 				},
 			})
 
@@ -196,7 +190,7 @@ var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 				Output: "",
 			})
 
-			_, err := config.InitConfig(ctx, fexec, nil)
+			_, err = config.InitConfig(ctx, fexec, nil)
 			Expect(err).NotTo(HaveOccurred())
 
 			stopChan := make(chan struct{})
@@ -214,7 +208,10 @@ var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 			return nil
 		}
 
-		err := app.Run([]string{app.Name})
+		err := app.Run([]string{
+			app.Name,
+			hoNodeCliArg,
+		})
 		Expect(err).NotTo(HaveOccurred())
 	})
 
@@ -253,19 +250,13 @@ var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 			Expect(err).NotTo(HaveOccurred())
 
 			Expect(fexec.CalledMatchesExpected()).To(BeTrue(), fexec.ErrorDesc)
-
-			expectedTables := map[string]util.FakeTable{
-				"filter": {
-					"INPUT": []string{"-p udp -m udp --dport 4789 -j ACCEPT"},
-				},
-				"nat": {},
-			}
-			Expect(ipt.MatchState(expectedTables)).NotTo(HaveOccurred())
-
 			return nil
 		}
 
-		err := app.Run([]string{app.Name})
+		err := app.Run([]string{
+			app.Name,
+			hoNodeCliArg,
+		})
 		Expect(err).NotTo(HaveOccurred())
 	})
 
@@ -283,7 +274,7 @@ var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 				Items: []v1.Node{
 					*createNode(node1Name, "windows", node1IP, map[string]string{
 						types.HybridOverlayNodeSubnet: node1Subnet,
-						types.HybridOverlayDrMac:      node1DrMAC,
+						types.HybridOverlayDRMAC:      node1DrMAC,
 					}),
 				},
 			})
@@ -294,8 +285,8 @@ var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 			fexec.AddFakeCmdsNoOutputNoError([]string{
 				"ovs-ofctl dump-flows br-ext table=0",
 				// Adds flows for existing node1
-				"ovs-ofctl add-flow br-ext cookie=0xca12f31b,table=0,priority=100,arp,in_port=ext,arp_tpa=" + node1Subnet + " actions=move:NXM_OF_ETH_SRC[]->NXM_OF_ETH_DST[],mod_dl_src:" + node1DrMAC + ",load:0x2->NXM_OF_ARP_OP[],move:NXM_NX_ARP_SHA[]->NXM_NX_ARP_THA[],load:0x" + node1DrMACRaw + "->NXM_NX_ARP_SHA[],move:NXM_OF_ARP_TPA[]->NXM_NX_REG0[],move:NXM_OF_ARP_SPA[]->NXM_OF_ARP_TPA[],move:NXM_NX_REG0[]->NXM_OF_ARP_SPA[],IN_PORT",
-				"ovs-ofctl add-flow br-ext cookie=0xca12f31b,table=0,priority=100,ip,nw_dst=5.6.7.0/24,actions=load:4097->NXM_NX_TUN_ID[0..31],set_field:10.0.0.2->tun_dst,set_field:22:33:44:55:66:77->eth_dst,output:1",
+				"ovs-ofctl add-flow br-ext cookie=0xca12f31b,table=0,priority=100,arp,in_port=ext,arp_tpa=" + node1Subnet + ",actions=move:NXM_OF_ETH_SRC[]->NXM_OF_ETH_DST[],mod_dl_src:" + node1DrMAC + ",load:0x2->NXM_OF_ARP_OP[],move:NXM_NX_ARP_SHA[]->NXM_NX_ARP_THA[],load:0x" + node1DrMACRaw + "->NXM_NX_ARP_SHA[],move:NXM_OF_ARP_TPA[]->NXM_NX_REG0[],move:NXM_OF_ARP_SPA[]->NXM_OF_ARP_TPA[],move:NXM_NX_REG0[]->NXM_OF_ARP_SPA[],IN_PORT",
+				"ovs-ofctl add-flow br-ext cookie=0xca12f31b,table=0,priority=100,ip,nw_dst=5.6.7.0/24,actions=load:4097->NXM_NX_TUN_ID[0..31],set_field:10.0.0.2->tun_dst,set_field:22:33:44:55:66:77->eth_dst,output:ext-vxlan",
 			})
 
 			_, err := config.InitConfig(ctx, fexec, nil)
@@ -317,7 +308,10 @@ var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 			return nil
 		}
 
-		err := app.Run([]string{app.Name})
+		err := app.Run([]string{
+			app.Name,
+			hoNodeCliArg,
+		})
 		Expect(err).NotTo(HaveOccurred())
 	})
 
@@ -338,13 +332,13 @@ var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 				Cmd: "ovs-ofctl dump-flows br-ext table=0",
 				// Return output for a stale node
 				Output: ` cookie=0x0, duration=137014.498s, table=0, n_packets=20, n_bytes=2605, priority=100,ip,in_port="ext-vxlan",dl_dst=00:00:00:4d:d9:c1,nw_dst=10.128.1.0/24 actions=resubmit(,10)
- cookie=0x1f40e27c, duration=61107.432s, table=0, n_packets=0, n_bytes=0, priority=100,arp,in_port=ext,arp_tpa=11.128.0.0/24 actions=move:NXM_OF_ETH_SRC[]->NXM_OF_ETH_DST[],mod_dl_src:00:00:00:33:65:d0,load:0x2->NXM_OF_ARP_OP[],move:NXM_NX_ARP_SHA[]->NXM_NX_ARP_THA[],load:0x3365d0->NXM_NX_ARP_SHA[],move:NXM_OF_ARP_TPA[]->NXM_NX_REG0[],move:NXM_OF_ARP_SPA[]->NXM_OF_ARP_TPA[],move:NXM_NX_REG0[]->NXM_OF_ARP_SPA[],IN_PORT
- cookie=0x1f40e27c, duration=61107.417s, table=0, n_packets=0, n_bytes=0, priority=100,ip,nw_dst=11.128.0.0/24 actions=load:0x1001->NXM_NX_TUN_ID[0..31],load:0xac110003->NXM_NX_TUN_IPV4_DST[],mod_dl_dst:00:00:00:33:65:d0,output:"ext-vxlan"
+ cookie=0x1f40e27c, duration=61107.432s, table=0, n_packets=0, n_bytes=0, priority=100,arp,in_port=ext,arp_tpa=10.132.0.0/24 actions=move:NXM_OF_ETH_SRC[]->NXM_OF_ETH_DST[],mod_dl_src:00:00:00:33:65:d0,load:0x2->NXM_OF_ARP_OP[],move:NXM_NX_ARP_SHA[]->NXM_NX_ARP_THA[],load:0x3365d0->NXM_NX_ARP_SHA[],move:NXM_OF_ARP_TPA[]->NXM_NX_REG0[],move:NXM_OF_ARP_SPA[]->NXM_OF_ARP_TPA[],move:NXM_NX_REG0[]->NXM_OF_ARP_SPA[],IN_PORT
+ cookie=0x1f40e27c, duration=61107.417s, table=0, n_packets=0, n_bytes=0, priority=100,ip,nw_dst=10.132.0.0/24 actions=load:0x1001->NXM_NX_TUN_ID[0..31],load:0xac110003->NXM_NX_TUN_IPV4_DST[],mod_dl_dst:00:00:00:33:65:d0,output:"ext-vxlan"
  cookie=0x0, duration=61107.658s, table=0, n_packets=50, n_bytes=3576, priority=0 actions=drop`,
 			})
 			fexec.AddFakeCmdsNoOutputNoError([]string{
 				// Deletes flows for stale node in OVS
-				"ovs-ofctl del-flows br-ext table=0, cookie=0x1f40e27c/0xffffffff",
+				"ovs-ofctl del-flows br-ext table=0,cookie=0x1f40e27c/0xffffffff",
 			})
 
 			_, err := config.InitConfig(ctx, fexec, nil)
@@ -366,7 +360,10 @@ var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 			return nil
 		}
 
-		err := app.Run([]string{app.Name})
+		err := app.Run([]string{
+			app.Name,
+			hoNodeCliArg,
+		})
 		Expect(err).NotTo(HaveOccurred())
 	})
 
@@ -388,7 +385,7 @@ var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 			})
 			fexec.AddFakeCmdsNoOutputNoError([]string{
 				// Deletes flows for pod in OVS that is not in Kube
-				"ovs-ofctl del-flows br-ext table=10, cookie=0xaabbccdd/0xffffffff",
+				"ovs-ofctl del-flows br-ext table=10,cookie=0xaabbccdd/0xffffffff",
 			})
 
 			_, err := config.InitConfig(ctx, fexec, nil)
@@ -437,7 +434,7 @@ var _ = Describe("Hybrid Overlay Node Linux Operations", func() {
 			})
 			fexec.AddFakeCmdsNoOutputNoError([]string{
 				// Refreshes flows for pod that is in OVS and in Kube
-				"ovs-ofctl add-flow br-ext table=10, cookie=0x7fdcde17, priority=100, ip, nw_dst=" + pod1CIDR + ", actions=set_field:" + hybMAC + "->eth_src,set_field:" + pod1MAC + "->eth_dst,output:ext",
+				"ovs-ofctl add-flow br-ext table=10,cookie=0x7fdcde17,priority=100,ip,nw_dst=" + pod1IP + ",actions=set_field:" + hybMAC + "->eth_src,set_field:" + pod1MAC + "->eth_dst,output:ext",
 			})
 
 			_, err := config.InitConfig(ctx, fexec, nil)
